@@ -15,8 +15,10 @@
 from django.conf import settings
 from django.contrib.sites.models import get_current_site, Site
 from django.contrib.auth.models import AnonymousUser
+from django.core.exceptions import ValidationError
+from django.db import IntegrityError
 from django.http import Http404
-from django.test import TestCase
+from django.test import TestCase as DjangoTestCase
 from django.test.client import RequestFactory
 from django_any.contrib import any_user
 from override_settings import override_settings
@@ -25,6 +27,63 @@ import mox
 from urlographer import models, utils, views
 
 
+class TestCase(DjangoTestCase):
+
+    def assertRaisesMessage(self, excClass, callableObj, excMessage,
+                            *args, **kwargs):
+        try:
+            callableObj(*args, **kwargs)
+        except excClass, e:
+            # message_dict and messages for Django's ValidationError
+            if hasattr(e, 'message_dict'):
+                e.message = e.message_dict
+            elif hasattr(e, 'messages'):
+                e.message = e.messages
+
+            if e.message != excMessage:
+                raise AssertionError(
+                    '%s message does not match %r, actual message: %r' %
+                    (excClass.__name__, excMessage, e.message))
+        else:
+            raise AssertionError('%s not raised' % excClass.__name__)
+
+
+class TestCaseTest(TestCase):
+    def raise_error(self, error_message, exc=ValidationError):
+        raise exc(error_message)
+
+    def test_raises_message_return_message(self):
+        self.assertRaisesMessage(
+            Exception, self.raise_error, 'test message',
+            error_message='test message', exc=Exception)
+
+    def test_raises_message_return_messages_list(self):
+        self.assertRaisesMessage(
+            ValidationError, self.raise_error, ['test message1', 'message2'],
+            error_message=['test message1', 'message2'])
+
+    def test_raises_message_return_message_dict(self):
+        self.assertRaisesMessage(
+            ValidationError, self.raise_error, {'field': 'test message1'},
+            error_message={'field': 'test message1'})
+
+    def test_raises_message_messages_dont_match(self):
+        # Assert that assertRaisesMessage raises AssertionError when messages
+        # don't match
+        self.assertRaises(
+            AssertionError,
+            self.assertRaisesMessage, ValidationError, self.raise_error,
+            ['test message1'], error_message=['message2'])
+
+    def test_raises_message_specified_exception_not_raised(self):
+        # Assert that assertRaisesMessage raises AssertionError when sprcified
+        # exception is not raised
+        self.assertRaisesMessage(
+            AssertionError,
+            self.assertRaisesMessage, 'ValidationError not raised',
+            ValidationError, str, 'test message')
+
+
 class URLMapTest(TestCase):
     def setUp(self):
         self.site = Site(domain='example.com')
@@ -72,20 +131,67 @@ class URLMapTest(TestCase):
         self.assertEqual(self.url.hexdigest, self.hexdigest)
         self.assertEqual(self.url.id, 1)
 
-    def test_save_invalid_redirect_raises(self):
+    def test_save_validates(self):
+        self.url.status_code = 204
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'site': [u'This field cannot be null.']})
+
+    def test_save_perm_redirect_wo_redirect_raises(self):
         self.site.save()
         self.url.site = self.site
         self.url.status_code = 301
-        self.assertRaises(AssertionError, self.url.save)
-        self.url.status_code = 302
-        self.assertRaises(AssertionError, self.url.save)
-        self.url.redirect = self.url
-        self.assertRaises(AssertionError, self.url.save)
-        self.url.status_code = 200
-        self.url.redirect = models.URLMap.objects.create(
-            site=self.site, path='/target', status_code=204)
-        self.assertRaises(AssertionError, self.url.save)
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'redirect': ['Status code requires a redirect']})
 
+    def test_save_temp_redirect_wo_redirect_raises(self):
+        self.site.save()
+        self.url.site = self.site
+        self.url.status_code = 302
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'redirect': ['Status code requires a redirect']})
+
+    def test_save_redirect_to_self_raises(self):
+        self.site.save()
+        self.url.site = self.site
+        self.url.status_code = 301
+        self.url.redirect = self.url
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'redirect': ['You cannot redirect a url to itself']})
+
+    def test_save_200_wo_content_map_raises(self):
+        self.site.save()
+        self.url.site = self.site
+        self.url.status_code = 200
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'content_map': ['Status code requires a content map']})
+
+    def test_delete_deletes_cache(self):
+        self.site.save()
+        self.url.site = self.site
+        self.url.status_code = 204
+        self.url.save()
+        self.mox.StubOutWithMock(models.cache, 'delete')
+        models.cache.delete(self.url.cache_key())
+        self.mox.ReplayAll()
+        self.url.delete()
+        self.mox.VerifyAll()
+        self.assertFalse(self.url.id)
+
+    def test_unique_hexdigest(self):
+        self.site.save()
+        self.url.site = self.site
+        self.url.status_code = 204
+        self.url.save()
+        self.url.id = None
+        self.assertRaisesMessage(
+            ValidationError, self.url.save,
+            {'hexdigest': [u'Url map with this Hexdigest already exists.']})
+        
 
 class URLMapManagerTest(TestCase):
     def setUp(self):
